package paystation.domain;

import org.junit.jupiter.api.Test;

import static org.hamcrest.CoreMatchers.*;
import static org.hamcrest.MatcherAssert.assertThat;

/** Integration testing the Omega Pay Station. */
public class TestOmegaPayStation {

  private PayStation ps;

  @Test
  public void shouldVerifyOmegaTownByIntegrationTesting()
          throws IllegalCoinException {
    // Given a Omega specification: when 1 $ has been
    // entered an immediate bonus of 25c is provided
    // by instantly increasing the 'insertedSoFar' from
    // 1$ to 1.25$. Each cent gives 2 minute parking
    RateStrategy rs = new OmegaRateStrategy();
    ps = new PayStationImpl(rs);
    // When entering 75 cent
    ps.addPayment(25);
    ps.addPayment(25);
    ps.addPayment(25);
    // Then 150 minutes is in the display
    assertThat(ps.readDisplay(), is(2*75));
    // When entering further 20 cent
    ps.addPayment(10); ps.addPayment(10);
    // Then 190 minutes is in the display
    assertThat(ps.readDisplay(), is(2*95));

    // When we enter 5 cent more, we pass the 1$ limit and
    ps.addPayment(5);
    // Then suddenly we have 250 minutes, and not 200 minutes
    assertThat(ps.readDisplay(), is(2*125));
  }

  @Test
  public void shouldAcceptLegalCoins() throws IllegalCoinException {
    // Given a pay station with a one cent gives one minute parking
    // rate strategy
    ps = new PayStationImpl(privatePayStation -> privatePayStation.getInsertedSoFar());
    // When I add 5+10+25 cents
    ps.addPayment( 5 );
    ps.addPayment( 10 );
    ps.addPayment( 25 );
    // Then I read that value in the display
    assertThat(ps.readDisplay(), is(5+10+25));
  }

  @Test
  public void shouldReturnCorrectReceiptWhenBuy()
          throws IllegalCoinException {
    // Given a pay station with a one cent gives one minute parking
    // rate strategy (using a method reference instead)
    ps = new PayStationImpl(ModifiablePayStation::getInsertedSoFar);
    // When I add 5+10+25 cents and buy parking
    ps.addPayment(5);
    ps.addPayment(10);
    ps.addPayment(25);
    Receipt receipt;
    receipt = ps.buy();
    // Then I get a receipt of that value
    assertThat(receipt.value(), is(5+10+25));
  }

}
