package paystation.domain;

import org.junit.jupiter.api.Test;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;

/** Unit testing the Omega Strategy by taking advantage of
 * the having a spy implementation of the private interface.
 */
public class TestOmegaStrategy {
  // A Test Spy implementation of the ModifiablePayStation role
  // A simple public string is used to record calls of the
  // adjustInsertedSoFar method.
  private class SpyModifiablePayStation
          implements ModifiablePayStation {
    private final int insertedSoFar;
    public String lastMethodCall = "not called";

    public SpyModifiablePayStation(int insertedSoFar) {
      this.insertedSoFar = insertedSoFar;
    }

    @Override
    public int getInsertedSoFar() {
      return insertedSoFar;
    }

    @Override
    public void adjustInsertedSoFar(int deltaValue) {
      lastMethodCall =
              "adjustInsertedSoFar("+deltaValue+") called.";
    }
  }

  @Test
  public void shouldUnitTestOmegaAdds25CentBonus() {
    // Given a spy on the ModifiablePayStation role which
    // returns 95 cents for insertedSoFar
    SpyModifiablePayStation spy = new SpyModifiablePayStation(95);
    // Given our omega rate strategy
    RateStrategy omegaStrategy = new OmegaRateStrategy();
    // When I call the calculate time method
    omegaStrategy.calculateTime(spy);
    // Then NO call has been made to the adjustInsertedSoFar method
    assertThat(spy.lastMethodCall, is("not called"));

    // Given a ModifiablePayStation role that does return 100 cents
    spy = new SpyModifiablePayStation(100);
    // When I call the calculate time
    omegaStrategy.calculateTime(spy);
    // Then one call has been made to the adjustMethodCall method
    // with new value of 125 for 'insertedSoFar'.
    assertThat(spy.lastMethodCall,
            is("adjustInsertedSoFar(125) called."));
  }
}
